@@ -2,7 +2,7 @@ package OpenTracing::Role::ContextReference;
 
 
 
-our $VERSION = '0.08_005';
+our $VERSION = 'v0.70.1';
 
 
 
@@ -11,7 +11,7 @@ use MooX::Enumeration;
 use MooX::ProtectedAttributes;
 
 use Types::Standard qw/Enum/;
-use OpenTracing::Types qw/SpanContext/;
+use Types::Interface qw/ObjectDoesInterface/;
 
 use constant CHILD_OF     => 'child_of';
 use constant FOLLOWS_FROM => 'follows_from';
@@ -27,7 +27,7 @@ protected_has reference_type => (
 
 has referenced_context => (
     is => 'ro',
-    isa => SpanContext,
+    isa => ObjectDoesInterface['OpenTracing::Interface::SpanContext'],
     required => 1,
     reader => 'get_referenced_context',
 );
