@@ -20,18 +20,14 @@ OpenTracing::Role::Scope - Role for OpenTracing implementations.
 
 
 
-our $VERSION = '0.08_005';
+our $VERSION = 'v0.70.1';
 
 
 
 use Moo::Role;
 
-use OpenTracing::Types qw/Span/;
-use Types::Standard qw/Bool CodeRef Maybe/;
-
-use Carp;
-
-
+use Types::Interface qw/ObjectDoesInterface/;
+use Types::Standard qw/Bool/;
 
 =head1 DESCRIPTION
 
@@ -44,7 +40,7 @@ L<OpenTracing::Interface>.
 
 has span => (
     is => 'ro',
-    isa => Span,
+    isa => ObjectDoesInterface['OpenTracing::Interface::Span'],
     reader => 'get_span',
 );
 
@@ -57,65 +53,28 @@ has finish_span_on_close => (
 
 
 
-has closed => (
-    is              => 'rwp',
-    isa             => Bool,
-    init_arg        => undef,
-    default         => !!undef,
-);
-
-
-
-has on_close => (
-    is              => 'ro',
-    isa             => Maybe[CodeRef],
-    predicate       => 1,
-);
-
-
-
-sub close {
+around close => sub {
+    my $orig = shift;
     my $self = shift;
     
-    croak "Can't close an already closed scope"
-        if $self->closed;
-    
-    $self->_set_closed( !undef );
+#   croak "Can't close an already closed scope"
+#       if $self->_has_closed;
+#   
+#   $self->_set_closed_time( epoch_floatingpoint() );
     
     $self->get_span->finish
         if $self->finish_span_on_close;
     
-    $self->on_close->( $self ) # we do like to have $self as invocant
-        if $self->has_on_close;
-    
+    $orig->( $self => @_ );
 #   return $self->get_scope_manager()->deactivate_scope( $self );
     
-    return $self
     
 };
 
 
 
-sub DEMOLISH {
-    my $self = shift;
-    my $in_global_destruction = shift;
-    
-    return if $self->closed;
-    
-    croak "Scope not programmatically closed before being demolished";
-    #
-    # below might be appreciated behaviour, but you should close yourself
-    #
-    $self->close( )
-        unless $in_global_destruction;
-    
-    return
-}
-
-
-
 BEGIN {
-    use Role::Tiny::With;
+#   use Role::Tiny::With;
     with 'OpenTracing::Interface::Scope'
         if $ENV{OPENTRACING_INTERFACE};
 }
