@@ -20,7 +20,7 @@ OpenTracing::Role::Span - Role for OpenTracing implementations.
 
 
 
-our $VERSION = '0.08_005';
+our $VERSION = 'v0.70.1';
 
 
 
@@ -28,11 +28,9 @@ use Moo::Role;
 use MooX::HandlesVia;
 
 use Carp;
-use Data::GUID;
-use Time::HiRes qw/time/;
-use Types::Standard qw/CodeRef HashRef Maybe Num Object Str Value/;
-use Types::Common::Numeric qw/PositiveOrZeroNum/;
-use OpenTracing::Types qw/:types :is/;
+use Time::HiRes qw/gettimeofday/;
+use Types::Standard qw/HashRef Num Object Str Value/;
+use Types::Interface qw/ObjectDoesInterface/;
 
 
 
@@ -49,28 +47,19 @@ leads to undefined behavior (but not returning an exception).
 
 
 
-has span_id => (
-    is              => 'ro',
-#   isa             => Uuid, # not restraints here, do so when consuming this
-    init_arg        => undef,
-    default         => sub { Data::GUID->new },
-);
-
-
-
 has operation_name => (
     is              => 'rwp',
     isa             => Str,
     required        => 1,
 #   writer          => 'overwrite_operation_name',
-    reader          => 'get_operation_name', # it's not in the Interface
+#   reader          => 'get_operation_name', # not sure, it's not in the Interface
 );
 
 
 
 has start_time => (
     is              => 'ro',
-    isa             => PositiveOrZeroNum,
+    isa             => Num,
     default         => sub { epoch_floatingpoint() }
 );
 
@@ -78,7 +67,7 @@ has start_time => (
 
 has finish_time => (
     is              => 'rwp',
-    isa             => PositiveOrZeroNum,
+    isa             => Num,
     predicate       => 'has_finished',
     init_arg        => undef,
 );
@@ -99,7 +88,7 @@ has tags => (
 
 has context => (
     is              => 'ro',
-    isa             => SpanContext,
+    isa             => ObjectDoesInterface['OpenTracing::Interface::SpanContext'],
     reader          => 'get_context',
 #   writer          => '_set_context',
     required        => 1, # either from Span->get_context or SpanContext self
@@ -125,15 +114,13 @@ sub overwrite_operation_name {
 sub finish {
     my $self = shift;
     
-    croak "Span has already been finished"
+    croak "Tag has already been finished"
         if $self->has_finished;
     
     my $epoch_timestamp = shift // epoch_floatingpoint();
     
     $self->_set_finish_time( $epoch_timestamp );
     
-    $self->on_finish->( $self ) # we do like to have $self as invocant
-        if $self->has_on_finish;
     return $self
 }
 
@@ -148,23 +135,7 @@ sub set_tag {
     my $key = shift;
     my $value = shift;
     
-    $self->set_tags( $key => $value );
-    
-    return $self
-}
-
-
-
-sub set_tags {
-    my $self = shift;
-    
-    croak "Can't set a tag on an already finished span"
-        if $self->has_finished;
-    
-    my %tags = @_;
-    $self->_set_tags(
-        { $self->get_tags, %tags }
-    );
+    $self->{ tags }->{ $key } = $value;
     
     return $self
 }
@@ -195,7 +166,8 @@ sub set_baggage_item {
     my $key = shift;
     my $value = shift;
     
-    $self->set_baggage_items( $key => $value );
+    my $new_context = $self->get_context()->with_baggage_item( $key, $value );
+    $self->_set_context( $new_context );
     
     return $self
 }
@@ -227,14 +199,6 @@ sub get_baggage_item {
 
 
 
-sub get_baggage_items {
-    my $self = shift;
-    
-    return $self->get_context()->get_baggage_items
-}
-
-
-
 sub duration { 
     my $self = shift;
     
@@ -242,14 +206,14 @@ sub duration {
         or croak
             "Span has not been started: ['"
             .
-            ( $self->get_operation_name || "'undef'" )
+            $self->operation_name
             .
             "'] ... how did you do that ?";
     my $finish_time = $self->{ finish_time }
         or croak
             "Span has not been finished: ['"
             .
-            ( $self->get_operation_name || "'undef'" )
+            $self->operation_name
             .
             "'] ... yet!";
     
@@ -258,29 +222,6 @@ sub duration {
 
 
 
-has child_of => (
-    is => 'ro',
-    isa =>Object, # does Span or does SpanContext
-    required => 1,
-);
-
-
-
-sub parent_span_id {
-    my $self = shift;
-    
-    my $parent = $self->{ child_of };
-    return unless is_Span( $parent );
-    
-    return $parent->span_id
-}
-#
-# This may not be the right way to implement it, for the `child_of` attribute
-# may not be such a good idea, maybe it should use references, but not sure how
-# those are used
-
-
-
 # _set_context
 #
 # you really shouldn't change the context yourself, only on instantiation
@@ -300,30 +241,6 @@ sub _set_context {
 
 
 
-has on_finish => (
-    is              => 'ro',
-    isa             => Maybe[CodeRef],
-    predicate       => 1,
-);
-
-
-
-sub DEMOLISH {
-    my $self = shift;
-    my $in_global_destruction = shift;
-    
-    return if $self->has_finished;
-    
-#   carp "Span not programmatically finished before being demolished";
-    
-    $self->finish( )
-        unless $in_global_destruction;
-    
-    return
-}
-
-
-
 =head2 epoch_floatingpoint
 
 Well, returns the time since 'epoch' with fractional seconds, as floating-point.
@@ -331,7 +248,7 @@ Well, returns the time since 'epoch' with fractional seconds, as floating-point.
 =cut
 
 sub epoch_floatingpoint {
-    return time()
+    return scalar gettimeofday()
 }
 #
 # well, this is a bit off a silly idea:
@@ -351,3 +268,5 @@ BEGIN {
 
 
 1;
+
+
