@@ -24,18 +24,15 @@ OpenTracing::Role::Tracer - Role for OpenTracin implementations.
 
 
 
-our $VERSION = '0.08_005';
+our $VERSION = 'v0.70.1';
 
 
 
 use Moo::Role;
 
 use Carp;
-use OpenTracing::Types qw/ScopeManager Span SpanContext/;
-use Role::Declare;
 use Try::Tiny;
-use Types::Common::Numeric qw/PositiveOrZeroNum/;
-use Types::Standard qw/HashRef Str/;
+use Types::Standard qw/Object/;
 
 
 
@@ -49,14 +46,17 @@ L<OpenTracing::Interface>.
 
 
 has scope_manager => (
-    is              => 'ro',
-    isa             => ScopeManager,
+    is              => 'lazy',
+    isa             => Object,
     reader          => 'get_scope_manager',
-    required        => 1,
 );
 
 
 
+requires '_build_scope_manager';
+
+requires 'build_span';
+
 requires 'extract_context';
 
 requires 'inject_context';
@@ -66,14 +66,16 @@ requires 'inject_context';
 sub get_active_span {
     my $self = shift;
     
-    my $scope_manager = $self->get_scope_manager
-        or croak "Can't get a 'ScopeManager'";
+    my $span = try {
+        $self->get_scope_manager->get_active_scope->get_span
+    } catch {
+        return undef
+    };
     
-    my $scope = $scope_manager->get_active_scope
-        or return;
-    
-    return $scope->get_span;
+    return $span
 }
+#
+# this is not really a good design but so convenient.
 
 
 
@@ -95,17 +97,9 @@ sub get_active_span_context {
 
 sub start_active_span {
     my $self = shift;
-    #
-    # TODO: croak when even number of arguments, after shifting invocant
-    #       first must be a operation name
-    #       then key / value pairs
-    
     my $operation_name = shift
         or croak "Missing required operation_name";
     my $opts = { @_ };
-    #
-    # TODO: only check on 'finish_span_on_close'
-    #       leave all other checks to 'start_span'
     
     # remove the `finish_span_on_close` option, which is for this method only! 
     my $finish_span_on_close = 
@@ -115,8 +109,6 @@ sub start_active_span {
     ; # use 'truthness' of param if provided, or set to 'true' otherwise
     
     my $span = $self->start_span( $operation_name => %$opts );
-    #
-    # TODO: use 'try { ... } catch { croak }'
     
     my $scope_manager = $self->get_scope_manager();
     
@@ -131,11 +123,6 @@ sub start_active_span {
 
 sub start_span {
     my $self = shift;
-    #
-    # TODO: croak when even number of arguments, after shifting invocant
-    #       first must be a operation name
-    #       then key / value pairs
-    
     my $operation_name = shift
         or croak "Missing required operation_name";
     my $opts = { @_ };
@@ -144,8 +131,6 @@ sub start_span {
     my $ignore_active_span = delete $opts->{ ignore_active_span };
     my $child_of           = delete $opts->{ child_of };
     my $tags               = delete $opts->{ tags };
-    #
-    # TODO: croak whit remaining options
     
     $child_of //= $self->get_active_span()
         unless $ignore_active_span;
@@ -155,20 +140,9 @@ sub start_span {
     my $context =
         defined $child_of
         && 
-#       $child_of->does('OpenTracing::Interface::SpanContext')
-        $child_of->can('with_baggage_item')
-        &&
-        $child_of->can('get_baggage_item')
-        #
-        # 'does' does not work, then just check on most essential methods
-        # TODO: use OpenTracing::Types
-        #
+        $child_of->does('OpenTracing::Interface::SpanContext')
         ?
         $child_of : $self->get_active_span_context();
-        #
-        # TODO: figure out why we want to get the active span it's context
-        #       what if we have passed in a span, should we take it's context?
-        #       is it more relevant to have the 'active context' at all times?
     
     my $span = $self->build_span(
         operation_name => $operation_name,
@@ -177,11 +151,6 @@ sub start_span {
         tags           => $tags,
         context        => $context,
     );
-    #
-    # we should get rid of passing 'child_of' or the not exisitng 'follows_from'
-    # these are merely helpers to define 'references'.
-    #
-    # TODO: use 'try { ... } catch { croak }'
     
     return $span
 }
@@ -200,15 +169,14 @@ The followin must be implemented by consuming class
 
 =cut
 
-instance_method build_span (
-    
-    Str                   :$operation_name,
-    SpanContext | Span    :$child_of,
-    SpanContext | HashRef :$context,
-    PositiveOrZeroNum     :$start_time      = undef,
-    HashRef[Str]          :$tags            = {},
-    
-) :Return (Span) { };
+##### requires 'build_span';
+#
+# $self->build_span(
+#     operartion_name => $operation_name,
+#     start_time      => $start_time,
+#     tags            => \@tags,
+#     context         => $context,
+# );
 
 
 
